---
title: $application实例化
---

这节分析入口文件最后两行代码，
```php
$application = $container->get(Hyperf\Contract\ApplicationInterface::class);
$application->run();
```
从容器中获取`ApplicationInterface`实例，然后调用`run`方法启动服务。

### $application是什么

从之前的分析可知，`$container`是容器类，从容器中解析出`ApplicationInterface`实例。
![](https://oss.xiaokeaii.top/2024/applicationInterface.png)
该接口对应的是`Hyperf\Framework\ApplicationFactory`类。分析一下取出源码，
```php
public function get($id)
{
    // 如果解析过，则直接返回
    if (isset($this->resolvedEntries[$id]) || array_key_exists($id, $this->resolvedEntries)) {
        return $this->resolvedEntries[$id];
    }
    return $this->resolvedEntries[$id] = $this->make($id);
}
```
上面可以看出，首次解析不走`if`，查看`make`方法，
```php
public function make(string $name, array $parameters = [])
{
    $definition = $this->getDefinition($name);
    if (! $definition) {
        throw new NotFoundException("No entry or class found for '{$name}'");
    }

    return $this->resolveDefinition($definition, $parameters);
}
```
前面绑定阶段，已经将`ApplicationInterface`绑定到`Hyperf\Framework\ApplicationFactory`类上，通过`DefinitionSource`类，将对应实现转换成对应`DefinitionInterface`接口的实现类。
```php
// 调用逻辑
public function __construct(array $source)
{
    $this->source = $this->normalizeSource($source);
}

protected function normalizeSource(array $source): array
{
    $definitions = [];
    foreach ($source as $identifier => $definition) {
        $normalizedDefinition = $this->normalizeDefinition($identifier, $definition);
        if (! is_null($normalizedDefinition)) {
            $definitions[$identifier] = $normalizedDefinition;
        }
    }
    return $definitions;
}

protected function normalizeDefinition(string $identifier, $definition): ?DefinitionInterface
{
    if ($definition instanceof PriorityDefinition) {
        $definition = $definition->getDefinition();
    }


    if (is_string($definition) && class_exists($definition)) {
        if (method_exists($definition, '__invoke')) {
            return new FactoryDefinition($identifier, $definition, []);
        }
        return $this->autowire($identifier, new ObjectDefinition($identifier, $definition));
    }

    if (is_callable($definition)) {
        return new FactoryDefinition($identifier, $definition, []);
    }

    return null;
}
```
查看`ApplicationFactory`类，存在`__invoke`方法，返回的是一个`FactoryDefinition`对象。
![](image https://oss.xiaokeaii.top/2024/applicationFactory.png)

```php
$definition = $this->getDefinition($name);
```
所以这行返回的是一个`FactoryDefinition`对象。打印结果如下，
![](image https://oss.xiaokeaii.top/2024/工厂定义.png	)

#### resolveDefinition方法
继续分析，查看`resolveDefinition`方法，
> 文件位置： /vendor/hyperf/di/src/Container.php
```php
protected function resolveDefinition(DefinitionInterface $definition, array $parameters = [])
{
    return $this->definitionResolver->resolve($definition, $parameters);
}
```
上面可以看到，这里调用了解析调度器`ResolverDispatcher`的`resolve`方法。

##### resolve方法
> 文件位置：/vendor/hyperf/di/src/Resolver/ResolverDispatcher.php
```php
public function resolve(DefinitionInterface $definition, array $parameters = [])
{
    if ($definition instanceof SelfResolvingDefinitionInterface) {

        return $definition->resolve($this->container);
    }
    $guard = DepthGuard::getInstance();

    return $guard->call(
        $definition->getName(),
        fn () => $this->getDefinitionResolver($definition)->resolve($definition, $parameters)
    );
}
```
这里`FactoryDefinition`不是`SelfResolvingDefinitionInterface`接口的实现类，所以不走`if`。
`$guard`是`DepthGuard`类的实例，调用`call`方法，
###### call方法
> 文件位置：/vendor/hyperf/di/src/Resolver/DepthGuard.php
```php
public function call(string $name, callable $callable)
{
    try {
        $this->increment();
        return $callable();
    } catch (CircularDependencyException $exception) {
        $exception->addDefinitionName($name);
        throw $exception;
    } finally {
        $this->decrement();
    }
}
```
这里最终执行传入的闭包，回到上面，执行闭包。
查看闭包执行源码，
> ./vendor/hyperf/di/src/Resolver/ResolverDispatcher.php
##### getDefinitionResolver方法
```php
private function getDefinitionResolver(DefinitionInterface $definition): ResolverInterface
{
    return match (true) {
        $definition instanceof ObjectDefinition => $this->objectResolver ??= new ObjectResolver($this->container, $this),
        $definition instanceof FactoryDefinition => $this->factoryResolver ??= new FactoryResolver($this->container, $this),
        default => throw new RuntimeException('No definition resolver was configured for definition of type ' . get_class($definition)),
    };
}
```
从代码可知，这里会根据`DefinitionInterface`的实现类，将对应解析器赋值给成员变量。
这里传进来的是`FactoryDefinition`类，所以会执行第二条。
紧接着调用`FactoryResolver`类的`resolve`方法。
###### resolve方法
> 文件位置：/vendor/hyperf/di/src/Resolver/FactoryResolver.php
```php
public function resolve(DefinitionInterface $definition, array $parameters = [])
{
    $callable = null;
    try {
        $callable = $definition->getFactory();
        if (! method_exists($callable, '__invoke')) {
            throw new NotCallableException();
        }
        if (is_string($callable)) {
            $callable = $this->container->get($callable);
        }
        return $callable($this->container, $parameters);
    } catch (NotCallableException $e) {
        // Custom error message to help debugging
        if (is_string($callable) && class_exists($callable) && method_exists($callable, '__invoke')) {
            throw new InvalidDefinitionException(sprintf('Entry "%s" cannot be resolved: factory %s. Invokable classes cannot be automatically resolved if autowiring is disabled on the container, you need to enable autowiring or define the entry manually.', $definition->getName(), $e->getMessage()));
        }

        throw new InvalidDefinitionException(sprintf('Entry "%s" cannot be resolved: factory %s', $definition->getName(), $e->getMessage()));
    }
}
```
分析该方法，`$callable`是工厂的类名，如果传递过来的是字符串，则通过容器解析出对应的类，方法同理，不再赘述。
最终调用该工厂类的`__invoke`方法返回解析结果。

### ApplicationFactory类
兜兜转转一圈，最终调用`ApplicationFactory`类的`__invoke`方法，获得解析后的对象。
```php
use Symfony\Component\Console\Application;
public function __invoke(ContainerInterface $container)
{
    if ($container->has(EventDispatcherInterface::class)) {
        $eventDispatcher = $container->get(EventDispatcherInterface::class);
        $eventDispatcher->dispatch(new BootApplication());
    }

    $config = $container->get(ConfigInterface::class);
    $commands = $config->get('commands', []);
    // 从注解中收集命令
    $annotationCommands = [];
    if (class_exists(AnnotationCollector::class) && class_exists(Command::class)) {
        $annotationCommands = AnnotationCollector::getClassesByAnnotation(Command::class);
        $annotationCommands = array_keys($annotationCommands);
    }

    $commands = array_unique(array_merge($commands, $annotationCommands));
    $application = new Application();

    if ($config->get('symfony.event.enable', false) && isset($eventDispatcher) && class_exists(SymfonyEventDispatcher::class)) {
        $application->setDispatcher(new SymfonyEventDispatcher($eventDispatcher));
    }
    // 将命令注册到应用中，下面会用到。
    foreach ($commands as $command) {
        $application->add(
            $this->pendingCommand($container->get($command))
        );
    }

    return $application;
}
```
从上面代码可以看出，返回的是一个`Symfony\Component\Console\Application`对象。

上面代码做了三件事：
1. 首先从容器中取出事件调度器，并触发`BootApplication`事件。
2. 创建`Application`应用对象，从注解和配置文件中获取所有命令。
3. 将命令注册到应用中。

#### add方法
查看`Application`类的`add`方法，

```php
public function add(Command $command)
{
    $this->init();

    $command->setApplication($this);

    if (!$command->isEnabled()) {
        $command->setApplication(null);

        return null;
    }

    if (!$command instanceof LazyCommand) {
        // Will throw if the command is not correctly initialized.
        $command->getDefinition();
    }

    if (!$command->getName()) {
        throw new LogicException(sprintf('The command defined in "%s" cannot have an empty name.', get_debug_type($command)));
    }
    // 重点看这行，将命令的名称以及对应实现添加到命令属性中
    $this->commands[$command->getName()] = $command;

    foreach ($command->getAliases() as $alias) {
        $this->commands[$alias] = $command;
    }
    return $command;
}
```
打印配置的命令数组，打开第一个`/vendor/hyperf/server/src/Command/StartServer.php`命令类，
![](image https://oss.xiaokeaii.top/2024/start.png)
> 我们可以发现，在父类的构造方法中，会将`name`赋值给该命令类的成员变量，之后通过`getName`方法获取命令的名称
```php
public function __construct(private ContainerInterface $container)
{
  parent::__construct('start');
  $this->setDescription('Start hyperf servers.');
}
// StartServer的父类的构造方法
public function __construct(?string $name = null)
{
  $this->definition = new InputDefinition();
  //...
  if (null !== $name) {
      $this->setName($name);
  }
  //...
}

public function getName(): ?string
{
  return $this->name;
}
```

### run启动
> 文件位置：/bin/hyperf.php
```php
$application->run();
```
从上面分析可知，`$application`是一个`Symfony\Component\Console\Application`对象，调用`run`方法。
先看源码，
```php
public function run(?InputInterface $input = null, ?OutputInterface $output = null): int
{
    // 设置终端的高度和宽度
    if (\function_exists('putenv')) {
        @putenv('LINES='.$this->terminal->getHeight());
        @putenv('COLUMNS='.$this->terminal->getWidth());
    }
    // 初始化输入和输出对象，如果没有提供输入和输出对象，则创建默认
    $input ??= new ArgvInput();
    $output ??= new ConsoleOutput();
    // 渲染异常的闭包函数
    $renderException = function (\Throwable $e) use ($output) {
        if ($output instanceof ConsoleOutputInterface) {
            $this->renderThrowable($e, $output->getErrorOutput());
        } else {
            $this->renderThrowable($e, $output);
        }
    };
    if ($phpHandler = set_exception_handler($renderException)) {
        restore_exception_handler();
        if (!\is_array($phpHandler) || !$phpHandler[0] instanceof ErrorHandler) {
            $errorHandler = true;
        } elseif ($errorHandler = $phpHandler[0]->setExceptionHandler($renderException)) {
            $phpHandler[0]->setExceptionHandler($errorHandler);
        }
    }
    // 根据输入参数和环境变量配置输入和输出对象
    $this->configureIO($input, $output);

    try {
        $exitCode = $this->doRun($input, $output);
    } catch (\Throwable $e) {
        if ($e instanceof \Exception && !$this->catchExceptions) {
            throw $e;
        }
        if (!$e instanceof \Exception && !$this->catchErrors) {
            throw $e;
        }

        $renderException($e);

        $exitCode = $e->getCode();
        if (is_numeric($exitCode)) {
            $exitCode = (int) $exitCode;
            if ($exitCode <= 0) {
                $exitCode = 1;
            }
        } else {
            $exitCode = 1;
        }
    } finally {
        // if the exception handler changed, keep it
        // otherwise, unregister $renderException
        if (!$phpHandler) {
            if (set_exception_handler($renderException) === $renderException) {
                restore_exception_handler();
            }
            restore_exception_handler();
        } elseif (!$errorHandler) {
            $finalHandler = $phpHandler[0]->setExceptionHandler(null);
            if ($finalHandler !== $renderException) {
                $phpHandler[0]->setExceptionHandler($finalHandler);
            }
        }
    }

    if ($this->autoExit) {
        if ($exitCode > 255) {
            $exitCode = 255;
        }

        exit($exitCode);
    }

    return $exitCode;
}

```
#### doRun方法
> 文件位置：/vendor/symfony/console/Application.php
```php
public function doRun(InputInterface $input, OutputInterface $output)
{
    // 输出版本信息
    if (true === $input->hasParameterOption(['--version', '-V'], true)) {
        $output->writeln($this->getLongVersion());

        return 0;
    }

    try {
        // Makes ArgvInput::getFirstArgument() able to distinguish an option from an argument.
        $input->bind($this->getDefinition());
    } catch (ExceptionInterface) {
        // Errors must be ignored, full binding/validation happens later when the command is known.
    }

    // 获取命令名称，项目启动代码是 php bin/hyperf.php start,
    // 所以命令名称就是 start
    $name = $this->getCommandName($input);
    // 如果命令名称是--help，则显示帮助信息
    if (true === $input->hasParameterOption(['--help', '-h'], true)) {
        if (!$name) {
            $name = 'help';
            $input = new ArrayInput(['command_name' => $this->defaultCommand]);
        } else {
            $this->wantHelps = true;
        }
    }
    // 如果命令名称不存在，则显示默认信息
    if (!$name) {
        $name = $this->defaultCommand;
        $definition = $this->getDefinition();
        $definition->setArguments(array_merge(
            $definition->getArguments(),
            [
                'command' => new InputArgument('command', InputArgument::OPTIONAL, $definition->getArgument('command')->getDescription(), $name),
            ]
        ));
    }

    try {
        $this->runningCommand = null;
        // 从应用中找到对应的命令
        $command = $this->find($name);
    } catch (\Throwable $e) {
        // 找不到的时候的异常处理
        if (($e instanceof CommandNotFoundException && !$e instanceof NamespaceNotFoundException) && 1 === \count($alternatives = $e->getAlternatives()) && $input->isInteractive()) {
            $alternative = $alternatives[0];

            $style = new SymfonyStyle($input, $output);
            $output->writeln('');
            $formattedBlock = (new FormatterHelper())->formatBlock(sprintf('Command "%s" is not defined.', $name), 'error', true);
            $output->writeln($formattedBlock);
            if (!$style->confirm(sprintf('Do you want to run "%s" instead? ', $alternative), false)) {
                if (null !== $this->dispatcher) {
                    $event = new ConsoleErrorEvent($input, $output, $e);
                    $this->dispatcher->dispatch($event, ConsoleEvents::ERROR);

                    return $event->getExitCode();
                }

                return 1;
            }

            $command = $this->find($alternative);
        } else {
            if (null !== $this->dispatcher) {
                $event = new ConsoleErrorEvent($input, $output, $e);
                $this->dispatcher->dispatch($event, ConsoleEvents::ERROR);

                if (0 === $event->getExitCode()) {
                    return 0;
                }

                $e = $event->getError();
            }

            try {
                if ($e instanceof CommandNotFoundException && $namespace = $this->findNamespace($name)) {
                    $helper = new DescriptorHelper();
                    $helper->describe($output instanceof ConsoleOutputInterface ? $output->getErrorOutput() : $output, $this, [
                        'format' => 'txt',
                        'raw_text' => false,
                        'namespace' => $namespace,
                        'short' => false,
                    ]);

                    return isset($event) ? $event->getExitCode() : 1;
                }

                throw $e;
            } catch (NamespaceNotFoundException) {
                throw $e;
            }
        }
    }
    // 如果commend继承LazyCommand类，则属于懒加载命令
    // 延迟实例化命令对象，以提高应用程序启动速度
    if ($command instanceof LazyCommand) {
        $command = $command->getCommand();
    }
    // 将运行的命令赋值给runningCommand属性
    $this->runningCommand = $command;
    // 执行命令
    $exitCode = $this->doRunCommand($command, $input, $output);
    $this->runningCommand = null;

    return $exitCode;
}
```

##### doRunCommand方法
这里正式进行命令的执行操作； 我们的启动命令是`start`，我们就以`start`命令为例，该命令对应的类是`StartServer`。

> 文件位置：/vendor/symfony/console/Application.php
```php
protected function doRunCommand(Command $command, InputInterface $input, OutputInterface $output)
{
    // 循环命令的帮助器集合，初始化时未配置，默认是有4个
    // return new HelperSet([
    //        new FormatterHelper(),
    //        new DebugFormatterHelper(),
    //        new ProcessHelper(),
    //        new QuestionHelper(),
    //    ]); 
    foreach ($command->getHelperSet() as $helper) {
        if ($helper instanceof InputAwareInterface) {
            $helper->setInput($input);
        }
    }
    // 获取当前命令是否实现了`SignalableCommandInterface`接口，如果是，获取订阅的信号列表
    // $commandSignals = $command instanceof SignalableCommandInterface ? $command->getSubscribedSignals() : [];
    // 检查是否存在需要处理的信号或者事件，$commandSignals初始化时为空，不处理信号
    // if ($commandSignals || $this->dispatcher && $this->signalsToDispatchEvent) {
    //     if (!$this->signalRegistry) {
    //         throw new RuntimeException('Unable to subscribe to signal events. Make sure that the "pcntl" extension is installed and that "pcntl_*" functions are not disabled by your php.ini\'s "disable_functions" directive.');
    //     }

    //     if (Terminal::hasSttyAvailable()) {
    //         $sttyMode = shell_exec('stty -g');

    //         foreach ([\SIGINT, \SIGTERM] as $signal) {
    //             $this->signalRegistry->register($signal, static fn () => shell_exec('stty '.$sttyMode));
    //         }
    //     }

    //     if ($this->dispatcher) {
    //         // We register application signals, so that we can dispatch the event
    //         foreach ($this->signalsToDispatchEvent as $signal) {
    //             $event = new ConsoleSignalEvent($command, $input, $output, $signal);

    //             $this->signalRegistry->register($signal, function ($signal) use ($event, $command, $commandSignals) {
    //                 $this->dispatcher->dispatch($event, ConsoleEvents::SIGNAL);
    //                 $exitCode = $event->getExitCode();

    //                 // If the command is signalable, we call the handleSignal() method
    //                 if (\in_array($signal, $commandSignals, true)) {
    //                     $exitCode = $command->handleSignal($signal, $exitCode);
    //                     // BC layer for Symfony <= 5
    //                     if (null === $exitCode) {
    //                         trigger_deprecation('symfony/console', '6.3', 'Not returning an exit code from "%s::handleSignal()" is deprecated, return "false" to keep the command running or "0" to exit successfully.', get_debug_type($command));
    //                         $exitCode = 0;
    //                     }
    //                 }

    //                 if (false !== $exitCode) {
    //                     $event = new ConsoleTerminateEvent($command, $event->getInput(), $event->getOutput(), $exitCode, $signal);
    //                     $this->dispatcher->dispatch($event, ConsoleEvents::TERMINATE);

    //                     exit($event->getExitCode());
    //                 }
    //             });
    //         }

    //         // then we register command signals, but not if already handled after the dispatcher
    //         $commandSignals = array_diff($commandSignals, $this->signalsToDispatchEvent);
    //     }

    //     foreach ($commandSignals as $signal) {
    //         $this->signalRegistry->register($signal, function (int $signal) use ($command): void {
    //             $exitCode = $command->handleSignal($signal);
    //             // BC layer for Symfony <= 5
    //             if (null === $exitCode) {
    //                 trigger_deprecation('symfony/console', '6.3', 'Not returning an exit code from "%s::handleSignal()" is deprecated, return "false" to keep the command running or "0" to exit successfully.', get_debug_type($command));
    //                 $exitCode = 0;
    //             }

    //             if (false !== $exitCode) {
    //                 exit($exitCode);
    //             }
    //         });
    //     }
    // }
    
    // 进入该方法
    if (null === $this->dispatcher) {
        return $command->run($input, $output);
    }

    // // bind before the console.command event, so the listeners have access to input options/arguments
    // try {
    //     $command->mergeApplicationDefinition();
    //     $input->bind($command->getDefinition());
    // } catch (ExceptionInterface) {
    //     // ignore invalid options/arguments for now, to allow the event listeners to customize the InputDefinition
    // }

    // $event = new ConsoleCommandEvent($command, $input, $output);
    // $e = null;

    // try {
    //     $this->dispatcher->dispatch($event, ConsoleEvents::COMMAND);

    //     if ($event->commandShouldRun()) {
    //         $exitCode = $command->run($input, $output);
    //     } else {
    //         $exitCode = ConsoleCommandEvent::RETURN_CODE_DISABLED;
    //     }
    // } catch (\Throwable $e) {
    //     $event = new ConsoleErrorEvent($input, $output, $e, $command);
    //     $this->dispatcher->dispatch($event, ConsoleEvents::ERROR);
    //     $e = $event->getError();

    //     if (0 === $exitCode = $event->getExitCode()) {
    //         $e = null;
    //     }
    // }

    // $event = new ConsoleTerminateEvent($command, $input, $output, $exitCode);
    // $this->dispatcher->dispatch($event, ConsoleEvents::TERMINATE);

    // if (null !== $e) {
    //     throw $e;
    // }

    // return $event->getExitCode();
}

```
查看`StartServer`中，没有`run`方法，查看父类`Command`，
> 文件位置： /vendor/symfony/console/Command/Command.php
```php
 public function run(InputInterface $input, OutputInterface $output): int
 {
     // add the application arguments and options
     $this->mergeApplicationDefinition();

     // bind the input against the command specific arguments/options
     try {
         $input->bind($this->getDefinition());
     } catch (ExceptionInterface $e) {
         if (!$this->ignoreValidationErrors) {
             throw $e;
         }
     }

     $this->initialize($input, $output);

    //  if (null !== $this->processTitle) {
    //      if (\function_exists('cli_set_process_title')) {
    //          if (!@cli_set_process_title($this->processTitle)) {
    //              if ('Darwin' === \PHP_OS) {
    //                  $output->writeln('<comment>Running "cli_set_process_title" as an unprivileged user is not supported on MacOS.</comment>', OutputInterface::VERBOSITY_VERY_VERBOSE);
    //              } else {
    //                  cli_set_process_title($this->processTitle);
    //              }
    //          }
    //      } elseif (\function_exists('setproctitle')) {
    //          setproctitle($this->processTitle);
    //      } elseif (OutputInterface::VERBOSITY_VERY_VERBOSE === $output->getVerbosity()) {
    //          $output->writeln('<comment>Install the proctitle PECL to be able to change the process title.</comment>');
    //      }
    //  }

    //  if ($input->isInteractive()) {
    //      $this->interact($input, $output);
    //  }

    //  // The command name argument is often omitted when a command is executed directly with its run() method.
    //  // It would fail the validation if we didn't make sure the command argument is present,
    //  // since it's required by the application.
    //  if ($input->hasArgument('command') && null === $input->getArgument('command')) {
    //      $input->setArgument('command', $this->getName());
    //  }

     $input->validate();

     if ($this->code) {
         $statusCode = ($this->code)($input, $output);
     } else {
        // 最终会执行execute方法
         $statusCode = $this->execute($input, $output);

         if (!\is_int($statusCode)) {
             throw new \TypeError(sprintf('Return value of "%s::execute()" must be of the type int, "%s" returned.', static::class, get_debug_type($statusCode)));
         }
     }

     return is_numeric($statusCode) ? (int) $statusCode : 0;
 }

```
分析上面代码可知，`run`方法最终会调用`execute`方法，继续往下看。
#### StartServer类
看了那么多代码，现在回到`StartServer`中的`execute`方法，
> 文件位置：/vendor/hyperf/server/src/Command/StartServer.php
```php
namespace Hyperf\Server\Command;

use Hyperf\Contract\ConfigInterface;
use Hyperf\Contract\StdoutLoggerInterface;
use Hyperf\Engine\Coroutine;
use Hyperf\Server\ServerFactory;
use Hyperf\Support\Composer;
use InvalidArgumentException;
use Psr\Container\ContainerInterface;
use Psr\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
//...

protected function execute(InputInterface $input, OutputInterface $output): int
{
    // 检查环境
    $this->checkEnvironment($output);
    // 第一步，取得ServerFactory实例对象
    $serverFactory = $this->container->get(ServerFactory::class)
        // 设置事件调度器
        ->setEventDispatcher($this->container->get(EventDispatcherInterface::class))
        // 设置日志
        ->setLogger($this->container->get(StdoutLoggerInterface::class));
    // 获取config目录下server.php中的server配置
    $serverConfig = $this->container->get(ConfigInterface::class)->get('server', []);

    if (! $serverConfig) {
        throw new InvalidArgumentException('At least one server should be defined.');
    }
    // 根据给定的参数配置服务
    $serverFactory->configure($serverConfig);

    Coroutine::set(['hook_flags' => swoole_hook_flags()]);

    $serverFactory->start();

    return 0;
}
```
> 默认情况下，配置文件中只有一个`http`服务。
##### configure方法
分析一下`configure`方法，
```php
public function configure(array $config): void
{
    // 创建一个serverConfig对象赋值给$this->config
    $this->config = new ServerConfig($config);
    //
    $this->getServer()->init($this->config);
}
// 看一下ServerConfig类的构造函数
public function __construct(protected array $config = [])
{
//   if (empty($config['servers'] ?? [])) {
//       throw new InvalidArgumentException('Config server.servers not exist.');
//   }

//   $servers = [];
//   foreach ($config['servers'] as $name => $item) {
//       if (! isset($item['name']) && ! is_numeric($name)) {
//           $item['name'] = $name;
//       }
//       $servers[] = Port::build($item);
//   }
  // 从这里可以看出，当配置文件中type不存在时，默认使用Hyperf\Server\Server类作为其类型
  $this->setType($config['type'] ?? Server::class)
      ->setMode($config['mode'] ?? 0)
      ->setServers($servers)
      ->setProcesses($config['processes'] ?? [])
      ->setSettings($config['settings'] ?? [])
      ->setCallbacks($config['callbacks'] ?? []);
}

public function start(): void
 {
     $this->getServer()->start();
 }

 public function getServer(): ServerInterface
 {
    // 首次进入，服务$this->server == null
     if (! $this->server instanceof ServerInterface) {
        // 这里默认的类型就为Hyperf\Server\Server类
         $serverName = $this->config->getType();
         // 实例化该Server类，传入日志和事件调度器
         $this->server = new $serverName(
             $this->container,
             $this->getLogger(),
             $this->getEventDispatcher()
         );
     }

     return $this->server;
 }
```
紧接着，执行`init`方法，
```php
// $this->getServer()返回的是一个Hyperf/Server/Server类实例
$this->getServer()->init($this->config);
```
##### Hyperf\Server\Server类
> 文件位置：/vendor/hyperf/server/src/Server.php
查看其`init`方法
```php
public function init(ServerConfig $config): ServerInterface
{
  $this->initServers($config);

  return $this;
}
 // 代码很长，先分析第一行
 protected function initServers(ServerConfig $config)
 {
     $servers = $this->sortServers($config->getServers());
     //...
 }

```
分析`sortServicers`方法，
```php
// $config->getServers()返回的是一个Hyperf\Server\Port类对象数组
$servers = $this->sortServers($config->getServers());
```
这里返回排好序的Port类对象数组。

之后循环遍历`$servers`数组，每个`value`是一个`Port`对象，走`if`
```php
foreach ($servers as $server) {
   $name = $server->getName();
   $type = $server->getType();
   $host = $server->getHost();
   $port = $server->getPort();
   $sockType = $server->getSockType();
   $callbacks = $server->getCallbacks();

   if (! $this->server instanceof SwooleServer) {
       // 分析makeServer方法
       // 这里返回的是一个Swoole\Server对象
       $this->server = $this->makeServer($type, $host, $port, $config->getMode(), $sockType);
       // 获取所有回调事件，进行合并
       $callbacks = array_replace($this->defaultCallbacks(), $config->getCallbacks(), $callbacks);
       // 注册回调事件
       $this->registerSwooleEvents($this->server, $callbacks, $name);
       // 设置运行参数
       $this->server->set(array_replace($config->getSettings(), $server->getSettings()));
       // 将服务以键值对的形式添加到ServerManager中
       ServerManager::add($name, [$type, current($this->server->ports)]);
       // 如果类存在，则触发BeforeMainServerStart事件
       if (class_exists(BeforeMainServerStart::class)) {
           // Trigger BeforeMainServerStart event, this event only trigger once before main server start.
           $this->eventDispatcher->dispatch(new BeforeMainServerStart($this->server, $config->toArray()));
       }
   } else {
       /** @var bool|\Swoole\Server\Port $slaveServer */
//        $slaveServer = $this->server->addlistener($host, $port, $sockType);
//        if (! $slaveServer) {
//            throw new \RuntimeException("Failed to listen server port [{$host}:{$port}]");
//        }
//        $server->getSettings() && $slaveServer->set(array_replace($config->getSettings(), $server->getSettings()));
//        $this->registerSwooleEvents($slaveServer, $callbacks, $name);
//        ServerManager::add($name, [$type, $slaveServer]);
   }

   // Trigger beforeStart event.
   if (isset($callbacks[Event::ON_BEFORE_START])) {
       [$class, $method] = $callbacks[Event::ON_BEFORE_START];
       if ($this->container->has($class)) {
           $this->container->get($class)->{$method}();
       }
   }

   if (class_exists(BeforeServerStart::class)) {
       // Trigger BeforeServerStart event.
       $this->eventDispatcher->dispatch(new BeforeServerStart($name));
   }
}
```
###### makeServer方法
分析可知，`makeServer`方法的作用就是根据`$type`参数，返回对应的`Server`类型。
根据配置文件中的`type`参数可知，返回的是一个`Swoole\Server`对象实例。
```php
use Swoole\Server as SwooleServer;

protected function makeServer(int $type, string $host, int $port, int $mode, int $sockType): SwooleServer
 {
     switch ($type) {
         case ServerInterface::SERVER_HTTP:
             return new SwooleHttpServer($host, $port, $mode, $sockType);
         case ServerInterface::SERVER_WEBSOCKET:
             return new SwooleWebSocketServer($host, $port, $mode, $sockType);
         case ServerInterface::SERVER_BASE:
             return new SwooleServer($host, $port, $mode, $sockType);
     }

     throw new RuntimeException('Server type is invalid.');
 }
```

###### registerSwooleEvents方法
看这一行，
```php
$this->registerSwooleEvents($this->server, $callbacks, $name);

protected function registerSwooleEvents(SwoolePort|SwooleServer $server, array $events, string $serverName): void
 {
     foreach ($events as $event => $callback) {
         // beforeStart不是swoole回调事件，会跳过
         if (! Event::isSwooleEvent($event)) {
             continue;
         }
         if (is_array($callback)) {
            // 解构出类名和方法名
             [$className, $method] = $callback;
             // 如果类名+方法名已经存在，追加到控制台输出警告信息
             if (array_key_exists($className . $method, $this->onRequestCallbacks)) {
                 $this->logger->warning(sprintf('%s will be replaced by %s. Each server should have its own onRequest callback. Please check your configs.', $this->onRequestCallbacks[$className . $method], $serverName));
             }
             // 追加到onRequestCallbacks数组中
             $this->onRequestCallbacks[$className . $method] = $serverName;
             // 从容器中解析出回调事件对象实例
             $class = $this->container->get($className);
             if (method_exists($class, 'setServerName')) {
                 // Override the server name.
                 $class->setServerName($serverName);
             }
             // 最后一个回调事件中，Hyperf\HttpServer\Server是MiddlewareInitializerInterface的实现类
             // 所以会进入该if
             // 这里会初始化中间件
             if ($class instanceof MiddlewareInitializerInterface) {
                 $class->initCoreMiddleware($serverName);
             }
             $callback = [$class, $method];
         }
         // 在server上注册回调事件
         $server->on($event, $callback);
     }
 }
```
这是所有的回调事件,如下图
![](image https://oss.xiaokeaii.top/2024/event.png	)

查看以下代码，这里进行配置和初始化中间件相关的内容
```php
$class->initCoreMiddleware($serverName);
```
> 文件位置：/vendor/hyperf/http-server/src/Server.php

```php
public function initCoreMiddleware(string $serverName): void
{
  // http
  $this->serverName = $serverName;
  // 创建核心中间件对象实例
  $this->coreMiddleware = $this->createCoreMiddleware();

  $config = $this->container->get(ConfigInterface::class);
  // 获取配置的中间件
  $this->middlewares = $config->get('middlewares.' . $serverName, []);
  // 设置异常处理器
  $this->exceptionHandlers = $config->get('exceptions.handler.' . $serverName, $this->getDefaultExceptionHandler());
  // 设置服务的option选项，对中间件进行排序
  $this->initOption();
}
```
逐行分析，
```php
protected function createCoreMiddleware(): CoreMiddlewareInterface
{
    return make(CoreMiddleware::class, [$this->container, $this->serverName]);
}
// make方法从容器中获取对象实例
function make(string $name, array $parameters = [])
{
    if (ApplicationContext::hasContainer()) {
        /** @var \Hyperf\Di\Container $container */
        $container = ApplicationContext::getContainer();
        if (method_exists($container, 'make')) {
            return $container->make($name, $parameters);
        }
    }
    $parameters = array_values($parameters);
    return new $name(...$parameters);
}

protected function initOption(): void
{
  $ports = $this->container->get(ServerFactory::class)->getConfig()?->getServers();
  if (! $ports) {
      return;
  }

  foreach ($ports as $port) {
      if ($port->getName() === $this->serverName) {
          $this->option = $port->getOptions();
      }
  }

  $this->option ??= Option::make([]);
  $this->option->setMustSortMiddlewaresByMiddlewares($this->middlewares);
}
```
###### ServerManager::add方法
上面分析完了，继续看下面的代码。
```php
// 设置sever运行时的参数
$this->server->set(array_replace($config->getSettings(), $server->getSettings()));
// 添加到ServerManager中
ServerManager::add($name, [$type, current($this->server->ports)]);
```

至此，`Hyperf\Server\Server`的`init`方法走完，返回自身实例。
`Hyperf\Server\ServerFactory`工厂类`configure`方法完成。
回到`StartServer`类的`execute`方法。


该方法的`$serverFactory->configure($serverConfig);`这一行执行完成。

```php
// 
Coroutine::set(['hook_flags' => swoole_hook_flags()]);
```

##### start方法
终于可以启动服务了，
> 文件位置：/vendor/hyperf/server/src/ServerFactory.php
```php
$serverFactory->start();
public function start(): void
{
  $this->getServer()->start();
}
```
`$this->getServer()`前面已经生成，这里直接返回`Hyperf\Server\Server`实例。
> 文件位置：/vendor/hyperf/server/src/Server.php
```php
public function start(): void
{
  $this->server->start();
}
```
其中`$this->server`对应的是`Swoole\Server`实例。调用`start`方法，实际就是启动`Swoole`的服务。
此时`server`启动，等待请求。